#!/usr/bin/env python3

# version: 2023-06-22

import sys, re, ruamel.yaml
from os import getlogin, path, system

dir_path = path.dirname( path.abspath(__file__) )
# pkg_path = path.join( dir_path, "bycon" )
# #sys.path.append( pkg_path )

from bycon import *

"""
The install script copies the relevant bycon files to the webserver directory
specified in the `config/config.yaml` file and sets the file permissions
accordingly. By default, it requires admin permissions (sudo). If you want to run
it without sudo, invoke it with `--no-sudo`. The current use will need to be able
to write into the target directories. 

Versions after 2023-02-07 _only_ copy the script directories and package __init__.py
to the server path; additionally a bycon package install is needed with
`pip3 install "bycon>=1.0.55"`.
"""

################################################################################
################################################################################
################################################################################

def main(no_sudo):

    install_beacon_server(no_sudo)

################################################################################
################################################################################
################################################################################

def install_beacon_server(no_sudo):
    if no_sudo:
        sudo_cmd = ""
    else:
        sudo_cmd = "sudo"

    yaml = ruamel.yaml.YAML()
    yaml.indent(mapping=2, sequence=4, offset=2)

    i_f = path.join( dir_path, "install.yaml" )
    try:
        with open( i_f ) as y_c:
            install = yaml.load( y_c )
    except Exception as e:
        print(e)
        exit()

    # WARNING: This assumes that the values are sensible...
    for p in ["system_user", "system_group", "bycon_install_dir"]:
        p_v = install.get(p, None)
        if p_v is None:
            print("¡¡¡ No `{}` value defined in {} !!!".format(p, i_f))
            exit()

    s_u = install["system_user"]
    s_g = install["system_group"]
    b_i_d_p = path.join( *install["bycon_install_dir"] )

    l_conf_source = path.join(dir_path, "local", "")
    server_source = path.join(dir_path, "bycon", "beaconServer", "")
    l_conf_target = path.join(server_source, "local", "")
    server_target = path.join(b_i_d_p, "beaconServer", "")

    system(f'{sudo_cmd} rsync -avh --delete {l_conf_source} {l_conf_target}')
    print(f'==> Copied configuration files from {l_conf_source} to {l_conf_target}')
    system(f'{sudo_cmd} rsync -avh --delete {server_source} {server_target}')
    print(f'==> Copied server files from {server_source} to {server_target}')

    system(f'{sudo_cmd} cp {path.join(dir_path, "bycon", "__init__.py")} {path.join(b_i_d_p, "__init__.py")}')
    system(f'{sudo_cmd} chown -R {s_u}:{s_g} {b_i_d_p}')
    system(f'{sudo_cmd} chmod 775 {server_target}*.py')
    print(f'{sudo_cmd} chmod 775 {server_target}*.py')

    print(f'Updated bycon files from\n{path.join(dir_path, "bycon")}\nto\n{b_i_d_p}')

################################################################################
################################################################################
################################################################################

if __name__ == '__main__':
    if len(sys.argv) > 1 and sys.argv[1] == "--no-sudo":
        no_sudo = True
    else:
        no_sudo = False

    main(no_sudo)



import datetime, re
from pymongo import MongoClient
from cgi_parsing import *
from interval_utils import interval_counts_from_callsets
from variant_mapping import ByconVariant
from variant_parsing import variant_state_from_variant_par
from os import environ

################################################################################
################################################################################
################################################################################

def reshape_resultset_results(ds_id, r_s_res, byc):
    r_s_res = remap_variants(r_s_res, byc)
    r_s_res = remap_analyses(r_s_res, byc)
    r_s_res = remap_biosamples(r_s_res, byc)
    r_s_res = remap_cohorts(r_s_res, byc)
    r_s_res = remap_individuals(r_s_res, byc)
    r_s_res = remap_phenopackets(ds_id, r_s_res, byc)
    r_s_res = remap_runs(r_s_res, byc)
    r_s_res = remap_all(r_s_res)

    return r_s_res


################################################################################

def remap_variants(r_s_res, byc):
    """
    Since the Beacon default model works from the concept of a canonical genomic
    variation and the bycon data model uses variant instances, the different
    instances of a "canonical" variant have to be identified and grouped together
    with individual instances indicated through their identifiers in `caseLevelData`.
    """

    if not "genomicVariant" in byc["response_entity_id"]:
        return r_s_res
    if "vcf" in byc["output"].lower() or "pgxseg" in byc["output"].lower():
        return r_s_res

    v_d = byc["variant_parameters"]

    variant_ids = []
    for v in r_s_res:
        variant_ids.append(v["variant_internal_id"])
    variant_ids = list(set(variant_ids))

    variants = []    

    for d in variant_ids:

        d_vs = [var for var in r_s_res if var.get('variant_internal_id', "__none__") == d]

        v = {
            "variant_internal_id": d,
            "variation": ByconVariant(byc).vrsVariant(d_vs[0]), "case_level_data": []
        }

        v["variation"].pop("variant_internal_id", None)

        for d_v in d_vs:
            v["case_level_data"].append(
                {
                    "id": d_v.get("id", "__none__"),
                    "biosample_id": d_v.get("biosample_id", "__none__"),
                    "analysis_id": d_v.get("callset_id", "__none__")
                }
            )

        # TODO: Keep legacy pars?
        legacy_pars = ["_id", "id", "reference_name", "type", "biosample_id", "callset_id", "individual_id",
                       "variant_type", "reference_bases", "alternate_bases", "start", "end"]
        for p in legacy_pars:
            v["variation"].pop(p, None)

        variants.append(v)

    return variants


################################################################################

def remap_analyses(r_s_res, byc):
    if not "analysis" in byc["response_entity_id"]:
        return r_s_res

    pop_keys = ["info", "provenance", "cnv_statusmaps", "cnv_chro_stats", "cnv_stats"]

    if "stats" in byc["output"]:
        pop_keys = ["info", "provenance", "cnv_statusmaps"]

    for cs_i, cs_r in enumerate(r_s_res):
        # TODO: REMOVE VERIFIER HACKS
        r_s_res[cs_i].update({"pipeline_name": "progenetix", "analysis_date": "1967-11-11"})
        for k in pop_keys:
            r_s_res[cs_i].pop(k, None)

    return r_s_res


################################################################################

def remap_cohorts(r_s_res, byc):
    if not "cohort" in byc["response_entity_id"]:
        return r_s_res

    cohorts = []

    # TODO: expand parameters by adding more pre-computed to collations (individual counts ...)

    for cs_i, cs_r in enumerate(r_s_res):
        cohorts.append({
            "cohort_id": cs_r.get("id", cs_i),
            "cohort_type": "beacon-defined",
            "cohort_name": cs_r.get("label", ""),
            "cohort_size": int(cs_r.get("count", 0))
        })

    return cohorts


################################################################################

def remap_runs(r_s_res, byc):
    if not "run" in byc["response_entity_id"]:
        return r_s_res

    runs = []
    for cs_i, cs_r in enumerate(r_s_res):
        r = {
            "id": cs_r.get("id", ""),
            "analysis_id": cs_r.get("id", ""),
            "biosample_id": cs_r.get("biosample_id", ""),
            "individual_id": cs_r.get("individual_id", ""),
            "run_date": datetime.datetime.fromisoformat(
                cs_r.get("updated", datetime.datetime.now().isoformat())).isoformat()
        }
        runs.append(r)

    return runs


################################################################################

def remap_biosamples(r_s_res, byc):
    if not "biosample" in byc["response_entity_id"]:
        return r_s_res

    bs_pop_keys = ["_id", "followup_state", "followup_time"]  # "info"

    for bs_i, bs_r in enumerate(r_s_res):

        # TODO: REMOVE VERIFIER HACKS
        r_s_res[bs_i].update({"sample_origin_type": {"id": "OBI:0001479", "label": "specimen from organism"}})

        for f in ["tumor_grade", "pathological_stage", "histological_diagnosis"]:
            try:
                if f in r_s_res[bs_i]:
                    if not r_s_res[bs_i][f]:
                        r_s_res[bs_i].pop(f, None)
                    elif not r_s_res[bs_i][f]["id"]:
                        r_s_res[bs_i].pop(f, None)
                    elif len(r_s_res[bs_i][f]["id"]) < 4:
                        r_s_res[bs_i].pop(f, None)
            except:
                pass

        for k in bs_pop_keys:
            r_s_res[bs_i].pop(k, None)

    return r_s_res


################################################################################

def remap_individuals(r_s_res, byc):
    if not "individual" in byc["response_entity_id"]:
        return r_s_res

    return r_s_res


################################################################################

def remap_phenopackets(ds_id, r_s_res, byc):
    if not "phenopacket" in byc["response_entity_id"]:
        return r_s_res

    mongo_client = MongoClient(host=environ.get("BYCON_MONGO_HOST", "localhost"))
    data_db = mongo_client[ds_id]
    pxf_s = []

    for ind_i, ind in enumerate(r_s_res):
        pxf = phenopack_individual(ind, data_db, byc)
        pxf_s.append(pxf)

    return pxf_s


################################################################################

def phenopack_individual(ind, data_db, byc):
    # TODO: key removal based on the ones not part of the respective PXF schemas
    # or better on filling them in only for existing parameters

    pxf_bios = []
    ind_pop_keys = ["_id", "provenance", "external_references", "description", "info"]
    bs_pop_keys = ["info", "provenance", "_id", "followup_time", "followup_state", "cohorts", "icdo_morphology",
                   "icdo_topography"]

    pxf_resources = _phenopack_resources(byc)
    server = select_this_server(byc)

    bios_s = data_db["biosamples"].find({"individual_id": ind["id"]})

    for bios in bios_s:

        bios.update({
            "files": [
                {
                    "uri": "{}/beacon/biosamples/{}/variants/?output=vcf".format(server, bios["id"]),
                    "file_attributes": {
                        "genomeAssembly": "GRCh38",
                        "fileFormat": "VCF"
                    }
                }
            ]
        })
        for k in bs_pop_keys:
            bios.pop(k, None)

        clean_empty_fields(bios)

        pxf_bios.append(bios)

    for k in ind_pop_keys:
        ind.pop(k, None)

    individual_remap_pgx_diseases(ind)

    for d_i, d in enumerate(ind["diseases"]):
        for k in ["followup_state", "followup_time"]:
            ind["diseases"][d_i].pop(k, None)

    pxf = {
        "id": re.sub("pgxind", "pgxpxf", ind["id"]),
        "subject": ind,
        "biosamples": pxf_bios,
        "metaData": {
            "submitted_by": "@mbaudis",
            "phenopacket_schema_version": "v2",
            "resources": pxf_resources
        }
    }

    return pxf


################################################################################

def clean_empty_fields(this_object):
    if not isinstance(this_object, dict):
        return this_object

    for k in list(this_object.keys()):
        if isinstance(this_object[k], dict):
            if not this_object[k]:
                this_object.pop(k, None)
        elif isinstance(this_object[k], list):
            if len(this_object[k]) < 1:
                this_object.pop(k, None)

    return this_object


################################################################################

def individual_remap_pgx_diseases(ind):
    # TODO: This should be more general, i.e. what is mapped/how, deleted...

    diseases = []

    for k in ["index_disease", "auxiliary_disease"]:
        try:
            if len(ind[k]["disease_code"]["id"]) > 1:
                diseases.append(ind[k])
        except:
            pass
        ind.pop(k, None)

    ind.update({"diseases": diseases})
    return ind


################################################################################

def _phenopack_resources(byc):
    # TODO: make this general, at least for phenopacket response, and only scan used prefixes

    f_d = byc["filter_definitions"]
    # rkeys = ["NCITgrade", "NCITstage", "NCITtnm", "NCIT", "PATOsex", "EFOfus" ]
    rkeys = ["NCIT", "PATOsex", "EFOfus", "UBERON"]

    pxf_rs = []

    for r_k in rkeys:
        if r_k not in f_d.keys():
            continue

        d = f_d[r_k]

        pxf_r = {
            "id": r_k,
            "name": d.get("name", ""),
            "url": d.get("url", ""),
            "namespace_prefix": d.get("namespace_prefix", ""),
            "version": "2022-04-01",
            "iri_prefix": "http://purl.obolibrary.org/obo/{}_".format(d.get("namespace_prefix", ""))
        }

        pxf_rs.append(pxf_r)

    return pxf_rs


################################################################################

def remap_all(r_s_res):
    for br_i, br_r in enumerate(r_s_res):
        if type(br_r) is not dict:
            continue
        r_s_res[br_i].pop("_id", None)
        clean_empty_fields(r_s_res[br_i])

    return r_s_res


################################################################################

def callset__ids_create_iset(ds_id, label, cs__ids, byc):
    mongo_client = MongoClient(host=environ.get("BYCON_MONGO_HOST", "localhost"))
    cs_coll = mongo_client[ds_id]["callsets"]

    cs_cursor = cs_coll.find({"_id": {"$in": cs__ids}, "variant_class": {"$ne": "SNV"}})
    intervals, cnv_cs_count = interval_counts_from_callsets(cs_cursor, byc)

    iset = {
        "dataset_id": ds_id,
        "group_id": ds_id,
        "label": label,
        "sample_count": cnv_cs_count,
        "interval_frequencies": []
    }

    for intv_i, intv in enumerate(intervals):
        iset["interval_frequencies"].append(intv.copy())

    return iset


################################################################################

def callsets_create_iset(ds_id, label, callsets, byc):
    intervals, cnv_cs_count = interval_counts_from_callsets(callsets, byc)

    iset = {
        "dataset_id": ds_id,
        "group_id": ds_id,
        "label": label,
        "sample_count": cnv_cs_count,
        "interval_frequencies": []
    }

    for intv_i, intv in enumerate(intervals):
        iset["interval_frequencies"].append(intv.copy())

    return iset



import re
from copy import deepcopy
from pydeepmerge import deep_merge

from cgi_parsing import prjsonnice
from datatable_utils import assign_nested_value, get_nested_value
from schema_parsing import object_instance_from_schema_name

################################################################################
################################################################################
################################################################################

class ByconVariant:

    def __init__(self, byc, variant={}):
        """
        # Class `ByconVariant`

        The class provides methods for the conversion ofgenomic variant objects
        with slight variations in their input parameters to a "canonical" intermediary
        `byc_variant` type, and from there then map per function to other formats
        (such as the Progenetix database format, VCF or VRS).

        The class is geared towards the data we currently process through the
        Progenetix platform and does not cover some use cases outside of Progenetix
        and Beacon "common use" scenarios (as of Beacon v2 / 2023).
        """

        self.byc = byc
        self.byc_variant = {}
        self.pgx_variant = {}
        self.vrs_variant = {}
        self.vcf_variant = {}
        self.pgxseg_variant = {}

        d_m = byc["datatable_mappings"].get("entities", {})
        d_m_v = d_m.get("genomicVariant", {})
        self.variant_mappings = d_m_v.get("parameters", {})

        self.vrs_allele = object_instance_from_schema_name(byc, "VRSallele", "")
        self.vrs_cnv = object_instance_from_schema_name(byc, "VRScopyNumberChange", "")

        if variant:
            self.byc_variant = deepcopy(variant)
            self.__create_canonical_variant()

    # -------------------------------------------------------------------------#
    # ----------------------------- public ------------------------------------#
    # -------------------------------------------------------------------------#

    def byconVariant(self, variant={}):
        if variant:
            self.byc_variant = deepcopy(variant)
            self.__create_canonical_variant()
        return self.byc_variant


    # -------------------------------------------------------------------------#

    def pgxVariant(self, variant={}):

        if variant:
            self.byc_variant = deepcopy(variant)
            self.__create_canonical_variant()
        b_v = self.byc_variant
        p_v = {} # to do: pgx cb variant instance
        for p_k, p_d in self.variant_mappings.items():
            dotted_key = p_d.get("db_key")
            i_v = b_v.get(p_k)
            if not dotted_key:
                continue
            assign_nested_value(p_v, dotted_key, i_v, p_d)

        self.pgx_variant.update(p_v) 
        return self.pgx_variant


    # -------------------------------------------------------------------------#

    def vcfVariant(self, variant={}):
        """
        Mapping of the relevant variant parameters into an object with

        * standard VCF column headers such as as `#CHROM` as keys
        * an `INFO` key + string for CNVs
        """
        if variant:
            self.byc_variant = deepcopy(variant)
            self.__create_canonical_variant()

        vt_defs = self.byc.get("variant_type_definitions", {})
        b_v = self.byc_variant

        # TODO: VCF schema in some config file...
        v_v = {
            "#CHROM": b_v.get("reference_name", "."),
            "POS": int(b_v.get("start", 0)) + 1,
            "ID": ".",
            "REF": b_v.get("reference_bases", "."),
            "ALT": b_v.get("alternate_bases", ""),
            "QUAL": ".",
            "FILTER": "PASS",
            "FORMAT": "GT",
            "INFO": ""
        }

        if len(v_v["REF"]) < 1:
             v_v.update({"REF": "."})

        v_s_id = b_v.get("variant_state_id", "___none___")
        if v_s_id in vt_defs.keys():
            s_a = vt_defs[v_s_id].get("VCF_symbolic_allele")
            if s_a and len(v_v["ALT"]) < 1:
                v_v.update({"ALT": s_a})

        v_l = b_v.get("variant_length", 1)
        if type(v_l) is int:
            if v_l >= 50:
                v_v.update({"INFO": f'IMPRECISE;SVCLAIM=D;END={b_v.get("end")};SVLEN={v_l}'})

        self.vcf_variant.update(v_v) 
        return self.vcf_variant


    # -------------------------------------------------------------------------#

    def vrsVariant(self, variant={}):
        """
        Mapping of the relevant variant parameters into a VRS object
        ... TODO ...
        """

        if variant:
            self.byc_variant = deepcopy(variant)
            self.__create_canonical_variant()

        vt_defs = self.byc.get("variant_type_definitions", {})
        v = self.byc_variant

        state_id = v.get("variant_state_id", "___none___")
        state_defs = vt_defs.get(state_id, {})
        vrs_type = state_defs.get("VRS_type", "___none___")

        if "Allele" in vrs_type:
            vrs_v = self.__vrs_allele()
        else:
            vrs_v = self.__vrs_cnv()

        # TODO: since the vrs_variant has been created as a new object we now
        #       add the annotation fields back (should epties be omitted?)
        for v_s in ("biosample_id", "callset_id", "id", "variant_internal_id"):
            vrs_v.update({v_s: v.get(v_s)})
        vrs_v.update({"variant_alternative_ids": v.get("variant_alternative_ids", [])})
        for v_o in ("identifiers", "info", "molecular_attributes", "variant_level_data"):
            vrs_v.update({v_o: v.get(v_o, {})})

        self.vrs_variant.update(vrs_v)
        return self.vrs_variant

    # -------------------------------------------------------------------------#

    def __vrs_allele(self):
        """
        A variant with a specified sequence as a subtype of VRS `MolecularVariation`.
        This remapping just covers the formats supported in the `bycon` environment
        but does not try to accommodate all use cases.
        """

        vt_defs = self.byc.get("variant_type_definitions", {})
        v = self.byc_variant

        vrs_a = deepcopy(self.vrs_allele)
        vrs_v = {
            "state": {
                "sequence": v.get("sequence")
            },
            "location": {
                "sequence_id": v.get("sequence_id"),
                "interval": {
                    "start": {"value": v.get("start")},
                    "end": {"value": v.get("end")}
                }
            }
        }

        vrs_v = deep_merge(vrs_a, vrs_v)

        return vrs_v


    # -------------------------------------------------------------------------#

    def __vrs_cnv(self):
        """
        A variant with a specified sequence as a subtype of VRS `SystemicVariation`.
        This remapping just covers the formats supported in the `bycon` environment
        but does not try to accommodate all use cases.
        """

        vt_defs = self.byc.get("variant_type_definitions", {})
        v = self.byc_variant

        vrs_c = deepcopy(self.vrs_cnv)
        vrs_v = {
            "copy_change": v.get("variant_state_id", "___none___").lower(),
            "subject": {
                "sequence_id": v.get("sequence_id"),
                "interval": {
                    "start": {"value": v.get("start")},
                    "end": {"value": v.get("end")}
                }
            }
        }
        vrs_v = deep_merge(vrs_c, vrs_v)

        return vrs_v


    # -------------------------------------------------------------------------#
    # ----------------------------- private -----------------------------------#
    # -------------------------------------------------------------------------#

    def __create_canonical_variant(self):
        vt_defs = self.byc.get("variant_type_definitions", {})
        g_aliases = self.byc.get("genome_aliases", {})

        v = self.byc_variant
        v.update({"errors": []})

        self.__byc_variant_normalize_type()
        self.__byc_variant_normalize_chromosome()
        self.__byc_variant_normalize_positions()
        self.__byc_variant_normalize_sequences()
        self.__byc_variant_add_digest()

        if not "info" in v:
            v.update({"info": {}})

        v.pop("variant_state", None)
        v.pop("location", None)

        # if self.byc["debug_mode"] is True:
        #     prjsonnice(v)

        return


    # -------------------------------------------------------------------------#

    def __byc_variant_normalize_type(self):
        vt_defs = self.byc.get("variant_type_definitions", {})
        v = self.byc_variant

        state_id = v.get("variant_state_id", "___none___")
        variant_type = v.get("variant_type", "___none___")
        if state_id not in vt_defs.keys():
            if "variant_state" in v:
                state_id = v["variant_state"].get("id", "___none___")

        if state_id not in vt_defs.keys():   
            if variant_type:
                for vt, vt_d in vt_defs.items():
                    dupdel = vt_d.get("DUPDEL", "___NA___")
                    tvt = vt_d.get("variant_type", "___NA___")
                    if dupdel == variant_type:
                        state_id = vt_d.get("dupdel_state_id")
                        if state_id in vt_defs.keys():
                            continue
                    elif tvt == variant_type:
                        state_id = vt_d.get("snv_state_id")
                        if state_id in vt_defs.keys():
                            continue

        if state_id in vt_defs.keys():
            state_defs = vt_defs[state_id]

            v.update({
                "variant_state_id": state_id,
                "variant_state_label": state_defs["variant_state"].get("label"),
                "variant_dupdel": state_defs.get("DUPDEL"),
                "variant_type": state_defs.get("variant_type"),
                "VCF_symbolic_allele": state_defs.get("VCF_symbolic_allele")
            })
        else:
            v["errors"].append(f'no variant type / state could be assigned')

        self.byc_variant.update(v)
        return


    # -------------------------------------------------------------------------#

    def __byc_variant_normalize_chromosome(self):
        g_aliases = self.byc.get("genome_aliases", {})
        v = self.byc_variant

        refs_ids = g_aliases.get("refseq_chronames", {}).keys()
        chro_ids = g_aliases.get("chro_refseq_ids", {}).keys()

        s_id = v.get("sequence_id", "___none___")
        chro = v.get("reference_name", "___none___")
        if s_id in refs_ids and chro in chro_ids:
            return

        # alternativ input parameters
        if s_id not in refs_ids:
            if "location" in v:
                s_id = v["location"].get("sequence_id", "___none___")

        if chro not in chro_ids:
            if "location" in v:
                chro = v["location"].get("chromosome", "___none___")

        if s_id in refs_ids and chro in chro_ids:
            pass
        elif s_id in refs_ids and chro not in chro_ids:
            c_a_s = g_aliases.get("chro_aliases", {})
            chro = c_a_s.get(s_id, "___none___")
        elif chro in chro_ids and s_id not in refs_ids:
            r_a_s = g_aliases.get("refseq_aliases", {})
            s_id = r_a_s.get(chro, "___none___")
        else:
            v["errors"].append(f'no sequence_id / chromosome could be assigned')

        v.update({
            "reference_name": chro,
            "sequence_id": s_id
        })

        self.byc_variant.update(v)
        return


    # -------------------------------------------------------------------------#

    def __byc_variant_normalize_sequences(self):
        v = self.byc_variant

        seq = v.get("sequence", v.get("alternate_bases"))
        r_seq = v.get("reference_sequence", v.get("reference__bases"))
        # TODO: check, normalize, default...
        v.update({
            "sequence": seq,
            "reference_sequence": r_seq        
        })

        self.byc_variant.update(v)
        return


    # -------------------------------------------------------------------------#

    def __byc_variant_normalize_positions(self):
        v = self.byc_variant
        if not "start" in v and "location" in v:
            v.update({
                "start": v["location"].get("start"),
                "end": v["location"].get("end")
            })

        try:
            v.update({"variant_length": v.get("end") - v.get("start")})
        except Exception as e:
            v["errors"].append(e)

        self.byc_variant.update(v)
        return


    # -------------------------------------------------------------------------#

    def __byc_variant_add_digest(self):
        v = self.byc_variant

        seq_re = '^[ACGTN]*$'
       
        v_lab = v.get("variant_type")
        seq = v.get("sequence", "")
        rseq = v.get("reference_sequence", "")
        if re.match(f'{seq_re}', str(seq)) or re.match(f'{seq_re}', str(rseq)):
            v_lab = f'{seq}>{rseq}'
        v.update({"variant_internal_id": f'{v["reference_name"]}:{v["start"]}-{v["end"]}:{v_lab}'})

        self.byc_variant.update(v)
        return

